package src.hw8Newton;

public class BusModel implements MathFunction2 {
    private double p  = 3.59;   // Density
    private double jj  = 0.05;   // Walk access speed
    private double k  = 0.4;    // Wait-hdwy ratio
    private double c  = 50;     // Bus cost
    private double v  = 0.167;  // Bus speed
    private double a0 = 0.41;   // Demand coefficients
    private double a2 = 0.0081;
    private double a4 = 0.0014;
    
    // Methods to return initial guesses for optimal values
    public double getRteSpacing() {
        return Math.pow((64*jj*jj*k*a4*c/(p*v*a0*a2)), 1.0/3.0);
    }
    
    public double getHeadway() {
        return Math.pow((a4*c/(p*a0*a2*k*k*jj*v)), 1.0/3.0);
    }
    
    public double getFare() {
        return (0.5*a0/a4 - Math.pow((k*c*a2*a2)/(jj*a4*a4*p*v*a0), 1.0/3.0));
    }
    
	@Override
	public double[] func(double[] x) {
		double[] f= new double[x.length];
		f[0]= -0.25*p*x[1]*a2/jj + 8*jj*k*c/(v*x[0]*x[0]*x[0]);
		f[1]= a0 - 0.5*a2*x[0]/jj - 2*a4*x[1];
		return f;
	}
	@Override
	public double[][] jacobian(double[] x) {
		int n= x.length;
		double[][] j= new double[n][n];
		j[0][0]= -24.0*jj*k*c/(v*x[0]*x[0]*x[0]*x[0]);
		j[0][1]= -0.25*p*a2/jj;
		j[1][0]= -0.5*a2/jj;
		j[1][1]= -2*a4;
		return j;
	}
}